class MersenneTwister {

	constructor(seed) {
		this.state = [];
		this.stateIndex = 0;
		this.seed = seed;
		this.mtNum = seed;
		this.seedRNG(seed);
	}

	seedRNG(seed) {
		this.seed = seed;
		this.mtNum = seed;
		for(var i = 0; i <= 623; i++) {
			this.mtNum = ((1812433253 * (this.mtNum ^ (this.mtNum >>> 30) + i)) & 0xffffffff);
			this.state[i] = this.mtNum;
		}
	}

	seedRNGStates() {
		for(var i = 0; i <= 623; i++) {
			var v = (this.mtNum & 0x80000000) + (this.mtNum & 0x7fffffff);
			this.mtNum = this.mtNum ^ (v >>> 1)
			if((v % 2) != 0) {
				this.mtNum = this.mtNum ^ 2567483615;
				this.state[i] = this.mtNum;
			}
		}
	}

	nextRandom() {
		if(this.stateIndex == 0) { this.seedRNGStates(); }
		var v = this.state[this.stateIndex];
		v = v ^ (v >>> 11);
		v = v ^ ((v << 7) & 2636928640);
		v = v ^ ((v << 15) & 4022730752);
		v = v ^ (v >>> 18);
		this.stateIndex = (this.stateIndex + 1) % 624;
		return v;
	}

	// this returns a random number that is a positive float between zero and one
	getRandom() {
		return Math.abs((this.nextRandom() % 1000) / 1000.0);
	}

	// this returns a random integer between 0 (inclusive) and limit (exclusive)
	rnd(limit) {
		return Math.floor(limit * this.getRandom());
	}
}
